(*----- Fonctions d'affichage d'un échiquier -----*)

(** Affiche un plateau (matrice). 
    Les cases occupées sont marquées par O, les autres d'un point. 
*)
let affiche_plateau p =
  let n = Array.length p in
  for i = 0 to n-1 do
    print_string " ";
    for j = 0 to n-1 do
      if p.(i).(j) then print_string "O "
      else print_string ". "
    done;
    print_char '\n'
  done


(** Affiche le plateau associé à une solution (partielle ou complète).
    sol est la liste des coos de la solution, n la dimension du damier.
*)
let affiche_solution sol n = 
  let echiquier = Array.make_matrix n n false in
  List.iter (fun (lgn,col) -> echiquier.(lgn).(col) <- true) sol;
  affiche_plateau echiquier



(* ----- Retour sur trace ----- *)

(* /!\ Contrairement au cours, on trouve avec des listes de PAIRES :
   on stocke pour chaque dame sa paire (ligne, colonne)
   On va s'efforcer de mettre une dame par ligne. *)

(** Teste si deux paires de coordonnées sont en prise. *)
let en_prise (i,j) (k,l) =
  i = k || j = l || (i-k = j-l) || (i-k = l-j)


(** Teste si la coordonnée de tête de sol est en prise 
    avec une des coordonnées de queue *)
let conflit_derniere sol =
  match sol with
  | h :: t -> List.exists (en_prise h) t
  | [] -> false
  

exception Found

(** Renvoie true si et seulement si les n dames admettent une solution *)
let soluble_n_dames n =

  (** Fonction récursive qui fait le retour sur trace.
      sol est la liste des dames déjà placées (de la dernière à la première)
      len_sol est le nombre d'éléments de sol.
  *)
  let rec retour_trace sol len_sol =
    if conflit_derniere sol then ()      (* abandon de cette branche *)
    else if len_sol = n then raise Found (* succès *)
    else begin                           (* on essaye toutes les possibilités *)
      for colonne = 0 to n-1 do
        retour_trace ((len_sol, colonne) :: sol) (len_sol +1)
      done
    end
  in 

  try
    retour_trace [] 0;
    false
  with Found -> true


(** Renvoie true si et seulement si les n dames admettent une solution *)
let compte_n_dames n =

  let total = ref 0 in

  (** Fonction récursive qui fait le retour sur trace.
      sol est la liste des dames déjà placées (de la dernière à la première)
      len_sol est le nombre d'éléments de sol.
  *)
  let rec retour_trace sol len_sol =
    if conflit_derniere sol then ()      (* abandon de cette branche *)
    else if len_sol = n then incr total  (* succès *)
    else begin                           (* on essaye toutes les possibilités *)
      for colonne = 0 to n-1 do
        retour_trace ((len_sol, colonne) :: sol) (len_sol +1)
      done
    end
  in 
  retour_trace [] 0;
  !total



let () =
  for i = 1 to 10 do
    Printf.printf "%d -> %d\n" i (compte_n_dames i)
  done
